"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.mapPersistedAccountInfos = exports.accountInfoFromPersisted = exports.loadAccount = exports.AccountPersister = void 0;
const web3_js_1 = require("@solana/web3.js");
const path_1 = __importDefault(require("path"));
const fs_1 = require("fs");
const fs_2 = require("../utils/fs");
const config_1 = require("../utils/config");
const assert_1 = require("assert");
const consts_1 = require("./consts");
class AccountPersister {
    constructor(targetDir, connection) {
        this.targetDir = targetDir;
        this.connection = connection;
    }
    // -----------------
    // Account Infos
    // -----------------
    async saveAccountInfo(address, accountInfo, subdir, label) {
        var _a;
        (0, assert_1.strict)(!accountInfo.executable, 'Can only save non-executable accounts');
        const fulldir = subdir ? path_1.default.join(this.targetDir, subdir) : this.targetDir;
        await (0, fs_2.ensureDir)(fulldir);
        const accountPath = path_1.default.join(fulldir, `${label !== null && label !== void 0 ? label : address.toBase58()}.json`);
        const persistedAccount = {
            pubkey: address.toBase58(),
            account: {
                lamports: accountInfo.lamports,
                data: [accountInfo.data.toString('base64'), 'base64'],
                owner: accountInfo.owner.toBase58(),
                executable: accountInfo.executable,
                rentEpoch: (_a = accountInfo.rentEpoch) !== null && _a !== void 0 ? _a : 0,
            },
        };
        await fs_1.promises.writeFile(accountPath, JSON.stringify(persistedAccount, null, 2));
        return accountPath;
    }
    async saveAccount(address, connection, data) {
        connection = this._requireConnection(connection, 'save Account');
        const accountInfo = await connection.getAccountInfo(address, 'confirmed');
        (0, assert_1.strict)(accountInfo != null, `Account not found at address ${address}`);
        if (data != null) {
            accountInfo.data = data;
        }
        return this.saveAccountInfo(address, accountInfo);
    }
    // -----------------
    // Keypairs
    // -----------------
    async saveKeypair(id, keypair, subdir) {
        const fulldir = subdir
            ? path_1.default.join(this.targetDir, subdir, consts_1.SNAPSHOT_KEYPAIRS_DIR)
            : path_1.default.join(this.targetDir, consts_1.SNAPSHOT_KEYPAIRS_DIR);
        await (0, fs_2.ensureDir)(fulldir);
        const keypairPath = path_1.default.join(fulldir, `${id}.json`);
        await fs_1.promises.writeFile(keypairPath, JSON.stringify(Buffer.from(keypair.secretKey).toJSON().data));
        return keypairPath;
    }
    // -----------------
    // Snapshot
    // -----------------
    async snapshot(snapshotLabel, addresses, 
    // Keyed pubkey:label
    accountLabels, keypairs, maybeConnection) {
        const snapshotRoot = this.targetDir;
        const connection = this._requireConnection(maybeConnection, 'take snapshot');
        const snapshotDir = path_1.default.join(snapshotRoot, snapshotLabel);
        await (0, fs_2.ensureDir)(snapshotDir, true);
        await Promise.all(addresses.map(async (address) => {
            const accountInfo = await connection.getAccountInfo(new web3_js_1.PublicKey(address), 'confirmed');
            if (accountInfo == null || accountInfo.executable)
                return;
            const subdir = path_1.default.join(snapshotLabel, consts_1.SNAPSHOT_ACCOUNTS_DIR);
            return this.saveAccountInfo(new web3_js_1.PublicKey(address), accountInfo, subdir, accountLabels[address]);
        }));
        await Promise.all(Array.from(keypairs.values()).map(async ({ keypair, id }) => {
            return this.saveKeypair(id, keypair, snapshotLabel);
        }));
        return snapshotDir;
    }
    _requireConnection(connection, task) {
        connection !== null && connection !== void 0 ? connection : (connection = this.connection);
        (0, assert_1.strict)(connection != null, `Must instantiate persister with connection or provide it to ${task}`);
        return connection;
    }
}
exports.AccountPersister = AccountPersister;
async function loadAccount(address, sourceDir, label) {
    const accountPath = path_1.default.join(sourceDir !== null && sourceDir !== void 0 ? sourceDir : (0, config_1.fullAccountsDir)(), `${label !== null && label !== void 0 ? label : address.toBase58()}.json`);
    const json = await fs_1.promises.readFile(accountPath, 'utf8');
    const persistedAccount = JSON.parse(json);
    return persistedAccount;
}
exports.loadAccount = loadAccount;
function accountInfoFromPersisted(persisted) {
    const { executable, owner, data, lamports } = persisted.account;
    assert_1.strict.equal(data[1], 'base64', 'expected persisted account info data to be encoded as "base64"');
    const accountInfo = {
        lamports,
        data: Buffer.from(data[0], 'base64'),
        owner: new web3_js_1.PublicKey(owner),
        executable,
    };
    return { address: persisted.pubkey, accountInfo };
}
exports.accountInfoFromPersisted = accountInfoFromPersisted;
function mapPersistedAccountInfos(persisteds) {
    const map = new Map();
    for (const persisted of persisteds) {
        const { address, accountInfo } = accountInfoFromPersisted(persisted);
        map.set(address, accountInfo);
    }
    return map;
}
exports.mapPersistedAccountInfos = mapPersistedAccountInfos;
//# sourceMappingURL=persistence.js.map