"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DisconnectedAmmanClient = exports.ConnectedAmmanClient = void 0;
const web3_js_1 = require("@solana/web3.js");
const assert_1 = require("assert");
const socket_io_client_1 = __importDefault(require("socket.io-client"));
const utils_1 = require("../utils");
const consts_1 = require("./consts");
const timeout_1 = require("./timeout");
const AMMAN_UNABLE_ADD_LABELS = 'Unable to connect to send address labels';
const AMMAN_UNABLE_FETCH_LABELS = 'Unable to connect to fetch address labels';
const AMMAN_UNABLE_FETCH_ACCOUNT_STATES = 'Unable to connect to fetch account states';
const AMMAN_UNABLE_SNAPSHOT_ACCOUNTS = 'Unable to connect to snapshot accounts';
const AMMAN_UNABLE_SAVE_ACCOUNT = 'Unable to connect to save account';
const AMMAN_UNABLE_STORE_KEYPAIR = 'Unable to connect to store keypair';
const AMMAN_UNABLE_LOAD_KEYPAIR = 'Unable to connect to load keypair';
const AMMAN_NOT_RUNNING_ERROR = ', is amman running with the relay enabled?\n' +
    'If not please start it as part of amman in a separate terminal via `amman start`\n' +
    'Make sure to set `relay: { enabled: true }` in `.ammanrc.js`.\n' +
    'Alternatively set environment var `CI=1` in your current terminal or\n' +
    'instantiate amman via `const amman = Amnnan.instance({ connectClient: false })`';
/** @private */
class ConnectedAmmanClient {
    constructor(url, opts = {}) {
        this.url = url;
        const { autoUnref = true, ack = false } = opts;
        this.ack = ack;
        this.socket = (0, socket_io_client_1.default)(url, { autoUnref });
    }
    connect() {
        if (this.socket.connected)
            return this;
        this.socket.connect();
        (0, utils_1.logDebug)('AmmanClient connected');
        return this;
    }
    clearAddressLabels() {
        // TODO(thlorenz): this should ack to resolve a promise
        this.socket.emit(consts_1.MSG_CLEAR_ADDRESS_LABELS);
    }
    clearTransactions() {
        // TODO(thlorenz): this should ack to resolve a promise
        this.socket.emit(consts_1.MSG_CLEAR_TRANSACTIONS);
    }
    addAddressLabels(labels) {
        if (utils_1.logTrace.enabled) {
            const labelCount = Object.keys(labels).length;
            (0, utils_1.logTrace)(`Adding ${labelCount} address labels`);
        }
        const promise = this.ack
            ? new Promise((resolve, reject) => {
                const timeout = (0, timeout_1.createTimeout)(2000, new Error(AMMAN_UNABLE_ADD_LABELS + AMMAN_NOT_RUNNING_ERROR), reject);
                this.socket
                    .on('error', (err) => {
                    clearTimeout(timeout);
                    reject(err);
                })
                    .on(consts_1.ACK_UPDATE_ADDRESS_LABELS, () => {
                    (0, utils_1.logTrace)('Got ack for address labels update %O', labels);
                    clearTimeout(timeout);
                    resolve();
                });
            })
            : Promise.resolve();
        this.socket.emit(consts_1.MSG_UPDATE_ADDRESS_LABELS, labels);
        return promise;
    }
    async fetchAddressLabels() {
        (0, utils_1.logTrace)('Fetching address labels');
        return new Promise((resolve, reject) => {
            const timeout = (0, timeout_1.createTimeout)(2000, new Error(AMMAN_UNABLE_FETCH_LABELS + AMMAN_NOT_RUNNING_ERROR), reject);
            this.socket
                .on('error', (err) => {
                clearTimeout(timeout);
                reject(err);
            })
                .on(consts_1.MSG_UPDATE_ADDRESS_LABELS, (labels) => {
                clearTimeout(timeout);
                (0, utils_1.logTrace)('Got address labels %O', labels);
                resolve(labels);
            })
                .emit(consts_1.MSG_GET_KNOWN_ADDRESS_LABELS);
        });
    }
    async fetchAccountStates(address) {
        (0, utils_1.logTrace)('Fetching account states for %s', address);
        return new Promise((resolve, reject) => {
            const timeout = (0, timeout_1.createTimeout)(2000, new Error(AMMAN_UNABLE_FETCH_ACCOUNT_STATES + AMMAN_NOT_RUNNING_ERROR), reject);
            this.socket
                .on('error', (err) => {
                clearTimeout(timeout);
                reject(err);
            })
                .on(consts_1.MSG_RESPOND_ACCOUNT_STATES, (accountAddress, states) => {
                clearTimeout(timeout);
                (0, utils_1.logDebug)('Got account states for address %s, %O', accountAddress, states);
                resolve(states);
            })
                .emit(consts_1.MSG_REQUEST_ACCOUNT_STATES, address);
        });
    }
    async requestSnapshot(label) {
        (0, utils_1.logTrace)('Requesting snapshot');
        label !== null && label !== void 0 ? label : (label = new Date().toJSON().replace(/[:.]/g, '_'));
        return new Promise((resolve, reject) => {
            const timeout = (0, timeout_1.createTimeout)(2000, new Error(AMMAN_UNABLE_SNAPSHOT_ACCOUNTS + AMMAN_NOT_RUNNING_ERROR), reject);
            this.socket
                .on('error', (err) => {
                clearTimeout(timeout);
                reject(err);
            })
                .on(consts_1.MSG_RESPOND_SNAPSHOT_SAVE, ({ err, snapshotDir }) => {
                clearTimeout(timeout);
                if (err != null)
                    return reject(new Error(err));
                (0, assert_1.strict)(snapshotDir != null, 'expected either error or snapshotDir');
                (0, utils_1.logDebug)('Completed snapshot at %s', snapshotDir);
                resolve(snapshotDir);
            })
                .emit(consts_1.MSG_REQUEST_SNAPSHOT_SAVE, label);
        });
    }
    async requestSaveAccount(address) {
        (0, utils_1.logTrace)('Requesting to save account "%s"', address);
        return new Promise((resolve, reject) => {
            const timeout = (0, timeout_1.createTimeout)(2000, new Error(AMMAN_UNABLE_SAVE_ACCOUNT + AMMAN_NOT_RUNNING_ERROR), reject);
            this.socket
                .on('error', (err) => {
                clearTimeout(timeout);
                reject(err);
            })
                .on(consts_1.MSG_RESPOND_ACCOUNT_SAVE, ({ err, accountPath }) => {
                clearTimeout(timeout);
                if (err != null)
                    return reject(new Error(err));
                (0, assert_1.strict)(accountPath != null, 'expected either error or accountPath');
                (0, utils_1.logDebug)('Completed saving account at %s', accountPath);
                resolve(accountPath);
            })
                .emit(consts_1.MSG_REQUEST_ACCOUNT_SAVE, address);
        });
    }
    async requestStoreKeypair(id, keypair) {
        (0, utils_1.logTrace)('Requesting to store keypair "%s" (%s)', id, keypair.publicKey.toBuffer());
        return new Promise((resolve, reject) => {
            const timeout = (0, timeout_1.createTimeout)(2000, new Error(AMMAN_UNABLE_STORE_KEYPAIR + AMMAN_NOT_RUNNING_ERROR), reject);
            this.socket
                .on('error', (err) => {
                clearTimeout(timeout);
                reject(err);
            })
                .on(consts_1.MSG_RESPOND_STORE_KEYPAIR, (err) => {
                clearTimeout(timeout);
                if (err != null)
                    return reject(new Error(err));
                resolve();
            })
                .emit(consts_1.MSG_REQUEST_STORE_KEYPAIR, id, keypair.secretKey);
        });
    }
    async requestLoadKeypair(id) {
        (0, utils_1.logTrace)('Requesting to load keypair with id "%s"', id);
        return new Promise((resolve, reject) => {
            const timeout = (0, timeout_1.createTimeout)(2000, new Error(AMMAN_UNABLE_LOAD_KEYPAIR + AMMAN_NOT_RUNNING_ERROR), reject);
            this.socket
                .on('error', (err) => {
                clearTimeout(timeout);
                reject(err);
            })
                .on(consts_1.MSG_RESPOND_LOAD_KEYPAIR, (secretKey) => {
                clearTimeout(timeout);
                try {
                    resolve(secretKey != null ? web3_js_1.Keypair.fromSecretKey(secretKey) : undefined);
                }
                catch (err) {
                    (0, utils_1.logError)('Failed to load keypair with id "%s"', id);
                    (0, utils_1.logError)(err);
                    resolve(undefined);
                }
            })
                .emit(consts_1.MSG_REQUEST_LOAD_KEYPAIR, id);
        });
    }
    /**
     * Disconnects this client and allows the app to shut down.
     * Only needed if you set `{ autoUnref: false }` for the opts.
     */
    disconnect() {
        this.socket.disconnect();
    }
    /**
     * Disconnects this client preventing reconnects and allows the app to shut
     * down. Only needed if you set `{ autoUnref: false }` for the opts.
     */
    destroy() {
        // @ts-ignore it' private
        if (typeof this.socket.destroy === 'function') {
            // @ts-ignore it' private
            this.socket.destroy();
        }
    }
    static getInstance(url, ammanClientOpts) {
        if (ConnectedAmmanClient._instance != null)
            return ConnectedAmmanClient._instance;
        ConnectedAmmanClient._instance = new ConnectedAmmanClient(url !== null && url !== void 0 ? url : consts_1.AMMAN_RELAY_URI, ammanClientOpts).connect();
        return ConnectedAmmanClient._instance;
    }
}
exports.ConnectedAmmanClient = ConnectedAmmanClient;
/** @private */
class DisconnectedAmmanClient {
    clearAddressLabels() { }
    clearTransactions() { }
    addAddressLabels(_labels) {
        return Promise.resolve();
    }
    fetchAddressLabels() {
        return Promise.resolve({});
    }
    fetchAccountStates(_address) {
        return Promise.resolve([]);
    }
    requestSnapshot(_label) {
        return Promise.resolve('');
    }
    requestSaveAccount(_address) {
        return Promise.resolve('');
    }
    requestStoreKeypair(_label, _keypair) {
        return Promise.resolve();
    }
    requestLoadKeypair(id) {
        return Promise.resolve(undefined);
    }
    disconnect() { }
    destroy() { }
}
exports.DisconnectedAmmanClient = DisconnectedAmmanClient;
//# sourceMappingURL=client.js.map