"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Relay = void 0;
const web3_js_1 = require("@solana/web3.js");
const http_1 = require("http");
const socket_io_1 = require("socket.io");
const providers_1 = require("../accounts/providers");
const state_1 = require("../accounts/state");
const assets_1 = require("../assets");
const utils_1 = require("../utils");
const http_2 = require("../utils/http");
const consts_1 = require("./consts");
const types_1 = require("./types");
/**
 * A simple socket.io server which communicates to the Amman Explorere as well as accepting connections
 * from other clients, i.e. via an {@link AmmanClient} which tests can use to communicate via the amman API.
 *
 * @private
 */
class RelayServer {
    constructor(io, accountProvider, accountPersister, snapshotPersister, accountStates, 
    // Keyed pubkey:label
    allKnownLabels = {}) {
        this.io = io;
        this.accountProvider = accountProvider;
        this.accountPersister = accountPersister;
        this.snapshotPersister = snapshotPersister;
        this.accountStates = accountStates;
        this.allKnownLabels = allKnownLabels;
        this.hookConnectionEvents();
    }
    hookConnectionEvents() {
        this.io.on('connection', (socket) => {
            const client = `${socket.id} from ${socket.client.conn.remoteAddress}`;
            socket.on('disconnect', () => (0, utils_1.logTrace)(`socket.io ${client} disconnected`));
            (0, utils_1.logTrace)(`socket.io ${client} connected`);
            this.hookMessages(socket);
        });
    }
    hookMessages(socket) {
        const subscribedAccountStates = new Set();
        socket
            .on(consts_1.MSG_UPDATE_ADDRESS_LABELS, (labels) => {
            if (utils_1.logTrace.enabled) {
                const labelCount = Object.keys(labels).length;
                (0, utils_1.logTrace)(`Got ${labelCount} labels, broadcasting ...`);
            }
            for (const [key, val] of Object.entries(labels)) {
                this.allKnownLabels[key] = val;
            }
            this.accountStates.labelKeypairs(this.allKnownLabels);
            socket.broadcast.emit(consts_1.MSG_UPDATE_ADDRESS_LABELS, labels);
            socket.emit(consts_1.ACK_UPDATE_ADDRESS_LABELS);
        })
            .on(consts_1.MSG_GET_KNOWN_ADDRESS_LABELS, () => {
            if (utils_1.logTrace.enabled) {
                const labelCount = Object.keys(this.allKnownLabels).length;
                (0, utils_1.logTrace)(`Sending ${labelCount} known labels to requesting client.`);
            }
            socket.emit(consts_1.MSG_UPDATE_ADDRESS_LABELS, this.allKnownLabels);
        })
            .on(consts_1.MSG_REQUEST_ACCOUNT_STATES, (pubkey) => {
            var _a;
            const states = (_a = this.accountStates.get(pubkey)) === null || _a === void 0 ? void 0 : _a.relayStates;
            if (states != null) {
                socket.emit(consts_1.MSG_RESPOND_ACCOUNT_STATES, pubkey, states);
            }
            if (!subscribedAccountStates.has(pubkey)) {
                subscribedAccountStates.add(pubkey);
                this.accountStates.on(`account-changed:${pubkey}`, (states) => {
                    socket.emit(consts_1.MSG_UPDATE_ACCOUNT_STATES, pubkey, states);
                });
            }
        })
            .on(consts_1.MSG_REQUEST_ACCOUNT_SAVE, async (pubkey, slot) => {
            try {
                let data;
                if (slot != null) {
                    data = this.accountStates.accountDataForSlot(pubkey, slot);
                }
                const accountPath = await this.accountPersister.saveAccount(new web3_js_1.PublicKey(pubkey), this.accountProvider.connection, data);
                socket.emit(consts_1.MSG_RESPOND_ACCOUNT_SAVE, pubkey, { accountPath });
            }
            catch (err) {
                socket.emit(consts_1.MSG_RESPOND_ACCOUNT_SAVE, pubkey, { err });
            }
        })
            .on(consts_1.MSG_REQUEST_SNAPSHOT_SAVE, async (label) => {
            try {
                const addresses = this.accountStates.allAccountAddresses();
                const snapshotDir = await this.snapshotPersister.snapshot(label, addresses, this.allKnownLabels, this.accountStates.allKeypairs);
                socket.emit(consts_1.MSG_RESPOND_SNAPSHOT_SAVE, { snapshotDir });
            }
            catch (err) {
                socket.emit(consts_1.MSG_RESPOND_SNAPSHOT_SAVE, { err: err.toString() });
            }
        })
            .on(consts_1.MSG_REQUEST_STORE_KEYPAIR, (id, secretKey) => {
            try {
                const keypair = web3_js_1.Keypair.fromSecretKey(secretKey);
                this.accountStates.storeKeypair(id, keypair);
                socket.emit(consts_1.MSG_RESPOND_STORE_KEYPAIR);
            }
            catch (err) {
                socket.emit(consts_1.MSG_RESPOND_STORE_KEYPAIR, err.toString());
            }
        })
            .on(consts_1.MSG_REQUEST_LOAD_KEYPAIR, (id) => {
            const keypair = this.accountStates.getKeypairById(id);
            socket.emit(consts_1.MSG_RESPOND_LOAD_KEYPAIR, keypair === null || keypair === void 0 ? void 0 : keypair.secretKey);
        })
            .on(consts_1.MSG_REQUEST_AMMAN_VERSION, () => {
            socket.emit(consts_1.MSG_RESPOND_AMMAN_VERSION, types_1.AMMAN_VERSION);
        });
    }
}
/**
 * Sets up the Amman Relay which uses the given account provider to resolve account data.
 * @private
 * */
class Relay {
    static createApp(accountProvider, accountPersister, snapshotPersister, accountStates, knownLabels) {
        const server = (0, http_1.createServer)();
        const io = new socket_io_1.Server(server, {
            cors: {
                origin: '*',
            },
        });
        const relayServer = new RelayServer(io, accountProvider, accountPersister, snapshotPersister, accountStates, knownLabels);
        return { app: server, io, relayServer };
    }
    static async startServer(accountProviders, accountRenderers, programs, accounts, loadedAccountInfos, loadedKeypairs, accountsFolder, snapshotRoot, killRunning = true) {
        if (killRunning) {
            await (0, http_2.killRunningServer)(consts_1.AMMAN_RELAY_PORT);
        }
        const accountProvider = providers_1.AccountProvider.fromRecord(accountProviders, accountRenderers);
        state_1.AccountStates.createInstance(accountProvider.connection, accountProvider, loadedAccountInfos, loadedKeypairs);
        const accountPersister = new assets_1.AccountPersister(accountsFolder, accountProvider.connection);
        const snapshotPersister = new assets_1.AccountPersister(snapshotRoot, accountProvider.connection);
        const programLabels = programs
            .filter((x) => x.label != null)
            .reduce((acc, x) => {
            acc[x.programId] = x.label;
            return acc;
        }, {});
        const accountLabels = accounts
            .filter((x) => x.label != null)
            .reduce((acc, x) => {
            acc[x.accountId] = x.label;
            return acc;
        }, {});
        const knownLabels = { ...programLabels, ...accountLabels };
        const { app, io, relayServer } = Relay.createApp(accountProvider, accountPersister, snapshotPersister, state_1.AccountStates.instance, knownLabels);
        return new Promise((resolve, reject) => {
            app.on('error', reject).listen(consts_1.AMMAN_RELAY_PORT, () => {
                const addr = app.address();
                const msg = `Amman Relay listening on ${addr.address}:${addr.port}`;
                (0, utils_1.logDebug)(msg);
                resolve({ app, io, relayServer });
            });
        });
    }
}
exports.Relay = Relay;
//# sourceMappingURL=server.js.map